@@ -23,6 +23,7 @@
 
 #include <KGuiItem>
 #include <KHistoryComboBox>
+#include <KLazyLocalizedString>
 #include <KLocalizedString>
 #include <KMessageBox>
 
@@ -118,7 +119,7 @@ void KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings,
     // If we have a selection, we make 'find in selection' default
     // and if we don't, then the option has to be unchecked, obviously.
     selectedText->setChecked(hasSelection);
-    _k_slotSelectedTextToggled(hasSelection);
+    slotSelectedTextToggled(hasSelection);
 
     promptOnReplace = new QCheckBox(i18n("&Prompt on replace"), optionGrp);
     promptOnReplace->setChecked(true);
@@ -133,8 +134,12 @@ void KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings,
 
     buttonBox = new QDialogButtonBox(q);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Close);
-    q->connect(buttonBox, SIGNAL(accepted()), q, SLOT(_k_slotOk()));
-    q->connect(buttonBox, SIGNAL(rejected()), q, SLOT(_k_slotReject()));
+    q->connect(buttonBox, &QDialogButtonBox::accepted, q, [this]() {
+        slotOk();
+    });
+    q->connect(buttonBox, &QDialogButtonBox::rejected, q, [this]() {
+        slotReject();
+    });
     topLayout->addWidget(buttonBox);
 
     // We delay creation of these until needed.
@@ -142,22 +147,30 @@ void KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings,
     placeholders = nullptr;
 
     // signals and slots connections
-    q->connect(selectedText, SIGNAL(toggled(bool)), q, SLOT(_k_slotSelectedTextToggled(bool)));
-    q->connect(regExp, &QAbstractButton::toggled, regExpItem, &QWidget::setEnabled);
-    q->connect(backRef, &QAbstractButton::toggled, backRefItem, &QWidget::setEnabled);
-    q->connect(regExpItem, SIGNAL(clicked()), q, SLOT(_k_showPatterns()));
-    q->connect(backRefItem, SIGNAL(clicked()), q, SLOT(_k_showPlaceholders()));
+    q->connect(selectedText, &QCheckBox::toggled, q, [this](bool checked) {
+        slotSelectedTextToggled(checked);
+    });
+    q->connect(regExp, &QCheckBox::toggled, regExpItem, &QWidget::setEnabled);
+    q->connect(backRef, &QCheckBox::toggled, backRefItem, &QWidget::setEnabled);
+    q->connect(regExpItem, &QPushButton::clicked, q, [this]() {
+        showPatterns();
+    });
+    q->connect(backRefItem, &QPushButton::clicked, q, [this]() {
+        showPlaceholders();
+    });
 
-    q->connect(find, SIGNAL(editTextChanged(QString)), q, SLOT(_k_textSearchChanged(QString)));
+    q->connect(find, &KHistoryComboBox::editTextChanged, q, [this](const QString &text) {
+        textSearchChanged(text);
+    });
 
-    q->connect(regExp, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(backRef, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(caseSensitive, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(wholeWordsOnly, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(fromCursor, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(findBackwards, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(selectedText, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
-    q->connect(promptOnReplace, &QAbstractButton::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(regExp, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(backRef, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(caseSensitive, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(wholeWordsOnly, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(fromCursor, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(findBackwards, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(selectedText, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
+    q->connect(promptOnReplace, &QCheckBox::toggled, q, &KFindDialog::optionsChanged);
 
     // tab order
     q->setTabOrder(find, regExp);
@@ -223,10 +236,10 @@ void KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings,
     findBackwards->setWhatsThis(i18n("Search backwards."));
     promptOnReplace->setWhatsThis(i18n("Ask before replacing each match found."));
 
-    _k_textSearchChanged(find->lineEdit()->text());
+    textSearchChanged(find->lineEdit()->text());
 }
 
-void KFindDialogPrivate::_k_textSearchChanged(const QString &text)
+void KFindDialogPrivate::textSearchChanged(const QString &text)
 {
     buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!text.isEmpty());
 }
@@ -331,11 +344,11 @@ void KFindDialog::setHasSelection(bool hasSelection)
     d->selectedText->setEnabled(hasSelection);
     if (!hasSelection) {
         d->selectedText->setChecked(false);
-        d->_k_slotSelectedTextToggled(hasSelection);
+        d->slotSelectedTextToggled(hasSelection);
     }
 }
 
-void KFindDialogPrivate::_k_slotSelectedTextToggled(bool selec)
+void KFindDialogPrivate::slotSelectedTextToggled(bool selec)
 {
     // From cursor doesn't make sense if we have a selection
     fromCursor->setEnabled(!selec && (enabled & KFind::FromCursor));
@@ -433,29 +446,29 @@ void KFindDialog::setOptions(long options)
 
 // Create a popup menu with a list of regular expression terms, to help the user
 // compose a regular expression search pattern.
-void KFindDialogPrivate::_k_showPatterns()
+void KFindDialogPrivate::showPatterns()
 {
     Q_Q(KFindDialog);
 
     typedef struct {
-        const char *description;
+        const KLazyLocalizedString description;
         const char *regExp;
         int cursorAdjustment;
     } Term;
     static const Term items[] = {
-        {I18N_NOOP("Any Character"), ".", 0},
-        {I18N_NOOP("Start of Line"), "^", 0},
-        {I18N_NOOP("End of Line"), "$", 0},
-        {I18N_NOOP("Set of Characters"), "[]", -1},
-        {I18N_NOOP("Repeats, Zero or More Times"), "*", 0},
-        {I18N_NOOP("Repeats, One or More Times"), "+", 0},
-        {I18N_NOOP("Optional"), "?", 0},
-        {I18N_NOOP("Escape"), "\\", 0},
-        {I18N_NOOP("TAB"), "\\t", 0},
-        {I18N_NOOP("Newline"), "\\n", 0},
-        {I18N_NOOP("Carriage Return"), "\\r", 0},
-        {I18N_NOOP("White Space"), "\\s", 0},
-        {I18N_NOOP("Digit"), "\\d", 0},
+        {kli18n("Any Character"), ".", 0},
+        {kli18n("Start of Line"), "^", 0},
+        {kli18n("End of Line"), "$", 0},
+        {kli18n("Set of Characters"), "[]", -1},
+        {kli18n("Repeats, Zero or More Times"), "*", 0},
+        {kli18n("Repeats, One or More Times"), "+", 0},
+        {kli18n("Optional"), "?", 0},
+        {kli18n("Escape"), "\\", 0},
+        {kli18n("TAB"), "\\t", 0},
+        {kli18n("Newline"), "\\n", 0},
+        {kli18n("Carriage Return"), "\\r", 0},
+        {kli18n("White Space"), "\\s", 0},
+        {kli18n("Digit"), "\\d", 0},
     };
 
     class RegExpAction : public QAction
@@ -492,7 +505,7 @@ void KFindDialogPrivate::_k_showPatterns()
     if (!patterns) {
         patterns = new QMenu(q);
         for (const Term &item : items) {
-            patterns->addAction(new RegExpAction(patterns, i18n(item.description), QLatin1String(item.regExp), item.cursorAdjustment));
+            patterns->addAction(new RegExpAction(patterns, item.description.toString(), QLatin1String(item.regExp), item.cursorAdjustment));
         }
     }
 
@@ -535,14 +548,16 @@ private:
 
 // Create a popup menu with a list of backreference terms, to help the user
 // compose a regular expression replacement pattern.
-void KFindDialogPrivate::_k_showPlaceholders()
+void KFindDialogPrivate::showPlaceholders()
 {
     Q_Q(KFindDialog);
 
     // Populate the popup menu.
     if (!placeholders) {
         placeholders = new QMenu(q);
-        q->connect(placeholders, SIGNAL(aboutToShow()), q, SLOT(_k_slotPlaceholdersAboutToShow()));
+        q->connect(placeholders, &QMenu::aboutToShow, q, [this]() {
+            slotPlaceholdersAboutToShow();
+        });
     }
 
     // Insert the selection into the edit control.
@@ -556,20 +571,20 @@ void KFindDialogPrivate::_k_showPlaceholders()
     }
 }
 
-void KFindDialogPrivate::_k_slotPlaceholdersAboutToShow()
+void KFindDialogPrivate::slotPlaceholdersAboutToShow()
 {
     Q_Q(KFindDialog);
 
     placeholders->clear();
     placeholders->addAction(new PlaceHolderAction(placeholders, i18n("Complete Match"), 0));
 
-    const int n = QRegularExpression(q->pattern()).captureCount();
+    const int n = QRegularExpression(q->pattern(), QRegularExpression::UseUnicodePropertiesOption).captureCount();
     for (int i = 1; i <= n; ++i) {
         placeholders->addAction(new PlaceHolderAction(placeholders, i18n("Captured Text (%1)", i), i));
     }
 }
 
-void KFindDialogPrivate::_k_slotOk()
+void KFindDialogPrivate::slotOk()
 {
     Q_Q(KFindDialog);
 
@@ -581,7 +596,7 @@ void KFindDialogPrivate::_k_slotOk()
 
     if (regExp->isChecked()) {
         // Check for a valid regular expression.
-        if (!QRegularExpression(q->pattern()).isValid()) {
+        if (!QRegularExpression(q->pattern(), QRegularExpression::UseUnicodePropertiesOption).isValid()) {
             KMessageBox::error(q, i18n("Invalid PCRE pattern syntax."));
             return;
         }
@@ -595,7 +610,7 @@ void KFindDialogPrivate::_k_slotOk()
     Q_EMIT q->okClicked();
 }
 
-void KFindDialogPrivate::_k_slotReject()
+void KFindDialogPrivate::slotReject()
 {
     Q_Q(KFindDialog);
 
