@@ -14,6 +14,7 @@
 #include "ktextwidgets_export.h"
 
 class QAction;
+class KRichTextWidgetPrivate;
 
 /**
  * @class KRichTextWidget krichtextwidget.h <KRichTextWidget>
@@ -40,7 +41,6 @@ class KTEXTWIDGETS_EXPORT KRichTextWidget : public KRichTextEdit
     Q_OBJECT
     Q_PROPERTY(RichTextSupport richTextSupport READ richTextSupport WRITE setRichTextSupport)
 public:
-
     /**
      * These flags describe what actions will be created by createActions() after
      * passing a combination of these flags to setRichTextSupport().
@@ -114,7 +114,7 @@ public:
 
         /**
          * Action to change the background color of the currently selected text. If no
-         * text is selected, the backgound color of the word under the cursor is
+         * text is selected, the background color of the word under the cursor is
          * changed.
          * Opens a QColorDialog to select the color.
          */
@@ -152,17 +152,17 @@ public:
          */
         FullListSupport = 0xf00,
 
-// Not implemented yet.
-//         SupportCreateTables = 0x1000,
-//         SupportChangeCellMargin = 0x2000,
-//         SupportChangeCellPadding = 0x4000,
-//         SupportChangeTableBorderWidth = 0x8000,
-//         SupportChangeTableBorderColor = 0x10000,
-//         SupportChangeTableBorderStyle = 0x20000,
-//         SupportChangeCellBackground = 0x40000,
-//         SupportCellFillPatterns = 0x80000,
-//
-//         FullTableSupport = 0xff000,
+        // Not implemented yet.
+        //         SupportCreateTables = 0x1000,
+        //         SupportChangeCellMargin = 0x2000,
+        //         SupportChangeCellPadding = 0x4000,
+        //         SupportChangeTableBorderWidth = 0x8000,
+        //         SupportChangeTableBorderColor = 0x10000,
+        //         SupportChangeTableBorderStyle = 0x20000,
+        //         SupportChangeCellBackground = 0x40000,
+        //         SupportCellFillPatterns = 0x80000,
+        //
+        //         FullTableSupport = 0xff000,
 
         /**
          * Actions to align the current paragraph left, righ, center or justify.
@@ -206,7 +206,7 @@ public:
          */
         SupportSuperScriptAndSubScript = 0x4000000,
 
-//         SupportChangeParagraphSpacing = 0x200000,
+        // SupportChangeParagraphSpacing = 0x200000,
 
         /**
          * Action to change direction of text to Right-To-Left or Left-To-Right.
@@ -227,7 +227,7 @@ public:
         /**
          * Includes all above actions for full rich text support
          */
-        FullSupport = 0xffffffff
+        FullSupport = 0xffffffff,
     };
     /**
      * Stores a combination of #RichTextSupportValues values.
@@ -253,7 +253,7 @@ public:
     /**
      * @brief Destructor
      */
-    ~KRichTextWidget();
+    ~KRichTextWidget() override;
 
     /**
      * @brief Creates the actions and adds them to the given action collection.
@@ -269,7 +269,7 @@ public:
      * Those names need to be the same in your XML file.
      *
      * See the KRichTextWidget::RichTextSupportValues enum documentation for a
-     * detailed explaination of each action.
+     * detailed explanation of each action.
      *
      * <table>
      * <tr><td><b>XML Name</b></td><td><b>RichTextSupportValues flag</b></td></tr>
@@ -350,15 +350,14 @@ protected:
 
 private:
     //@cond PRIVATE
-    class Private;
-    friend class Private;
-    Private *const d;
-    Q_PRIVATE_SLOT(d, void _k_updateCharFormatActions(const QTextCharFormat &))
-    Q_PRIVATE_SLOT(d, void _k_updateMiscActions())
+    Q_DECLARE_PRIVATE_D(KTextEdit::d, KRichTextWidget)
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 79)
+    // Unused, kept for ABI compatibility
+    const void *__ktextwidgets_d_do_not_use;
+#endif
     //@endcond
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KRichTextWidget::RichTextSupport)
 
 #endif
-
