@@ -1,166 +1,52 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2002 Carsten Pfeiffer <pfeiffer@kde.org>
-                 2005 Michael Brade <brade@kde.org>
-         2012 Laurent Montel <montel@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2002 Carsten Pfeiffer <pfeiffer@kde.org>
+    SPDX-FileCopyrightText: 2005 Michael Brade <brade@kde.org>
+    SPDX-FileCopyrightText: 2012 Laurent Montel <montel@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "ktextedit.h"
+#include "ktextedit_p.h"
 
 #include <QAction>
+#include <QActionGroup>
 #include <QApplication>
 #include <QClipboard>
+#include <QDebug>
 #include <QKeyEvent>
 #include <QMenu>
 #include <QScrollBar>
 #include <QTextCursor>
-#include <QTextDocumentFragment>
-#include <QDebug>
-#ifdef HAVE_SPEECH
-#include <QTextToSpeech>
-#endif
-#include <QSettings>
 
+#include <KCursor>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KStandardAction>
+#include <KStandardShortcut>
+#include <sonnet/backgroundchecker.h>
 #include <sonnet/configdialog.h>
 #include <sonnet/dialog.h>
-#include <sonnet/backgroundchecker.h>
-#include <sonnet/spellcheckdecorator.h>
-#include <kcursor.h>
-#include <kstandardaction.h>
-#include <kstandardshortcut.h>
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
-
-#include "kreplacedialog.h"
-#include "kfinddialog.h"
-#include "kfind.h"
-#include "kreplace.h"
 
 class KTextDecorator : public Sonnet::SpellCheckDecorator
 {
 public:
     explicit KTextDecorator(KTextEdit *textEdit);
     bool isSpellCheckingEnabledForBlock(const QString &textBlock) const override;
+
 private:
     KTextEdit *m_textEdit;
 };
 
-class Q_DECL_HIDDEN KTextEdit::Private
+void KTextEditPrivate::checkSpelling(bool force)
 {
-public:
-    Private(KTextEdit *_parent)
-        : parent(_parent),
-          languagesMenu(nullptr),
-          customPalette(false),
-          spellCheckingEnabled(false),
-          findReplaceEnabled(true),
-          showTabAction(true),
-          showAutoCorrectionButton(false),
-          decorator(nullptr), speller(nullptr), findDlg(nullptr), find(nullptr), repDlg(nullptr), replace(nullptr),
-#ifdef HAVE_SPEECH
-          textToSpeech(nullptr),
-#endif
-          findIndex(0), repIndex(0),
-          lastReplacedPosition(-1)
-    {
-        //Check the default sonnet settings to see if spellchecking should be enabled.
-        QSettings settings(QStringLiteral("KDE"), QStringLiteral("Sonnet"));
-        spellCheckingEnabled = settings.value(QStringLiteral("checkerEnabledByDefault"), false).toBool();
-    }
+    Q_Q(KTextEdit);
 
-    ~Private()
-    {
-        delete decorator;
-        delete findDlg;
-        delete find;
-        delete replace;
-        delete repDlg;
-        delete speller;
-#ifdef HAVE_SPEECH
-        delete textToSpeech;
-#endif
-    }
-
-    /**
-     * Checks whether we should/should not consume a key used as a shortcut.
-     * This makes it possible to handle shortcuts in the focused widget before any
-     * window-global QAction is triggered.
-     */
-    bool overrideShortcut(const QKeyEvent *e);
-    /**
-     * Actually handle a shortcut event.
-     */
-    bool handleShortcut(const QKeyEvent *e);
-
-    void spellCheckerMisspelling(const QString &text, int pos);
-    void spellCheckerCorrected(const QString &, int, const QString &);
-    void spellCheckerAutoCorrect(const QString &, const QString &);
-    void spellCheckerCanceled();
-    void spellCheckerFinished();
-    void toggleAutoSpellCheck();
-
-    void slotFindHighlight(const QString &text, int matchingIndex, int matchingLength);
-    void slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength);
-
-    /**
-     * Similar to QTextEdit::clear(), only that it is possible to undo this
-     * action.
-     */
-    void undoableClear();
-
-    void slotAllowTab();
-    void menuActivated(QAction *action);
-
-    void init();
-
-    void checkSpelling(bool force);
-    KTextEdit *parent;
-    QAction *autoSpellCheckAction;
-    QAction *allowTab;
-    QAction *spellCheckAction;
-    QMenu *languagesMenu;
-    bool customPalette : 1;
-
-    bool spellCheckingEnabled : 1;
-    bool findReplaceEnabled: 1;
-    bool showTabAction: 1;
-    bool showAutoCorrectionButton: 1;
-    QTextDocumentFragment originalDoc;
-    QString spellCheckingLanguage;
-    Sonnet::SpellCheckDecorator *decorator;
-    Sonnet::Speller *speller;
-    KFindDialog *findDlg;
-    KFind *find;
-    KReplaceDialog *repDlg;
-    KReplace *replace;
-#ifdef HAVE_SPEECH
-    QTextToSpeech *textToSpeech;
-#endif
-
-    int findIndex, repIndex;
-    int lastReplacedPosition;
-};
-
-void KTextEdit::Private::checkSpelling(bool force)
-{
-    if (parent->document()->isEmpty()) {
-        KMessageBox::information(parent, i18n("Nothing to spell check."));
+    if (q->document()->isEmpty()) {
+        KMessageBox::information(q, i18n("Nothing to spell check."));
         if (force) {
-            emit parent->spellCheckingFinished();
+            Q_EMIT q->spellCheckingFinished();
         }
         return;
     }
@@ -168,90 +54,103 @@ void KTextEdit::Private::checkSpelling(bool force)
     if (!spellCheckingLanguage.isEmpty()) {
         backgroundSpellCheck->changeLanguage(spellCheckingLanguage);
     }
-    Sonnet::Dialog *spellDialog = new Sonnet::Dialog(
-        backgroundSpellCheck, force ? parent : nullptr);
+    Sonnet::Dialog *spellDialog = new Sonnet::Dialog(backgroundSpellCheck, force ? q : nullptr);
     backgroundSpellCheck->setParent(spellDialog);
     spellDialog->setAttribute(Qt::WA_DeleteOnClose, true);
     spellDialog->activeAutoCorrect(showAutoCorrectionButton);
-    connect(spellDialog, SIGNAL(replace(QString,int,QString)),
-            parent, SLOT(spellCheckerCorrected(QString,int,QString)));
-    connect(spellDialog, SIGNAL(misspelling(QString,int)),
-            parent, SLOT(spellCheckerMisspelling(QString,int)));
-    connect(spellDialog, &Sonnet::Dialog::autoCorrect,
-            parent, &KTextEdit::spellCheckerAutoCorrect);
-    connect(spellDialog, SIGNAL(done(QString)),
-            parent, SLOT(spellCheckerFinished()));
-    connect(spellDialog, SIGNAL(cancel()),
-            parent, SLOT(spellCheckerCanceled()));
-    //Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
-    /*
-    connect(spellDialog, SIGNAL(stop()),
-            parent, SLOT(spellCheckerFinished()));
-    */
-    connect(spellDialog, &Sonnet::Dialog::spellCheckStatus,
-            parent, &KTextEdit::spellCheckStatus);
-    connect(spellDialog, &Sonnet::Dialog::languageChanged,
-            parent, &KTextEdit::languageChanged);
+    QObject::connect(spellDialog, &Sonnet::Dialog::replace, q, [this](const QString &oldWord, int pos, const QString &newWord) {
+        spellCheckerCorrected(oldWord, pos, newWord);
+    });
+    QObject::connect(spellDialog, &Sonnet::Dialog::misspelling, q, [this](const QString &text, int pos) {
+        spellCheckerMisspelling(text, pos);
+    });
+    QObject::connect(spellDialog, &Sonnet::Dialog::autoCorrect, q, &KTextEdit::spellCheckerAutoCorrect);
+    QObject::connect(spellDialog, &Sonnet::Dialog::spellCheckDone, q, [this]() {
+        spellCheckerFinished();
+    });
+    QObject::connect(spellDialog, &Sonnet::Dialog::cancel, q, [this]() {
+        spellCheckerCanceled();
+    });
+
+    // Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
+    // connect(spellDialog, SIGNAL(stop()), q, SLOT(spellCheckerFinished()));
+
+    QObject::connect(spellDialog, &Sonnet::Dialog::spellCheckStatus, q, &KTextEdit::spellCheckStatus);
+    QObject::connect(spellDialog, &Sonnet::Dialog::languageChanged, q, &KTextEdit::languageChanged);
     if (force) {
-        connect(spellDialog, SIGNAL(done(QString)), parent, SIGNAL(spellCheckingFinished()));
-        connect(spellDialog, &Sonnet::Dialog::cancel, parent, &KTextEdit::spellCheckingCanceled);
-        //Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
-        //connect(spellDialog, SIGNAL(stop()), parent, SIGNAL(spellCheckingFinished()));
+        QObject::connect(spellDialog, &Sonnet::Dialog::spellCheckDone, q, &KTextEdit::spellCheckingFinished);
+        QObject::connect(spellDialog, &Sonnet::Dialog::cancel, q, &KTextEdit::spellCheckingCanceled);
+        // Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
+        // connect(spellDialog, SIGNAL(stop()), q, SIGNAL(spellCheckingFinished()));
     }
-    originalDoc = QTextDocumentFragment(parent->document());
-    spellDialog->setBuffer(parent->toPlainText());
+    originalDoc = QTextDocumentFragment(q->document());
+    spellDialog->setBuffer(q->toPlainText());
     spellDialog->show();
 }
 
-void KTextEdit::Private::spellCheckerCanceled()
+void KTextEditPrivate::spellCheckerCanceled()
 {
-    QTextDocument *doc = parent->document();
+    Q_Q(KTextEdit);
+
+    QTextDocument *doc = q->document();
     doc->clear();
     QTextCursor cursor(doc);
     cursor.insertFragment(originalDoc);
     spellCheckerFinished();
 }
 
-void KTextEdit::Private::spellCheckerAutoCorrect(const QString &currentWord, const QString &autoCorrectWord)
+void KTextEditPrivate::spellCheckerAutoCorrect(const QString &currentWord, const QString &autoCorrectWord)
 {
-    emit parent->spellCheckerAutoCorrect(currentWord, autoCorrectWord);
+    Q_Q(KTextEdit);
+
+    Q_EMIT q->spellCheckerAutoCorrect(currentWord, autoCorrectWord);
 }
 
-void KTextEdit::Private::spellCheckerMisspelling(const QString &text, int pos)
+void KTextEditPrivate::spellCheckerMisspelling(const QString &text, int pos)
 {
-    //qDebug()<<"TextEdit::Private::spellCheckerMisspelling :"<<text<<" pos :"<<pos;
-    parent->highlightWord(text.length(), pos);
+    Q_Q(KTextEdit);
+
+    // qDebug()<<"TextEdit::Private::spellCheckerMisspelling :"<<text<<" pos :"<<pos;
+    q->highlightWord(text.length(), pos);
 }
 
-void KTextEdit::Private::spellCheckerCorrected(const QString &oldWord, int pos, const QString &newWord)
+void KTextEditPrivate::spellCheckerCorrected(const QString &oldWord, int pos, const QString &newWord)
 {
-    //qDebug()<<" oldWord :"<<oldWord<<" newWord :"<<newWord<<" pos : "<<pos;
+    Q_Q(KTextEdit);
+
+    // qDebug()<<" oldWord :"<<oldWord<<" newWord :"<<newWord<<" pos : "<<pos;
     if (oldWord != newWord) {
-        QTextCursor cursor(parent->document());
+        QTextCursor cursor(q->document());
         cursor.setPosition(pos);
         cursor.setPosition(pos + oldWord.length(), QTextCursor::KeepAnchor);
         cursor.insertText(newWord);
     }
 }
 
-void KTextEdit::Private::spellCheckerFinished()
+void KTextEditPrivate::spellCheckerFinished()
 {
-    QTextCursor cursor(parent->document());
+    Q_Q(KTextEdit);
+
+    QTextCursor cursor(q->document());
     cursor.clearSelection();
-    parent->setTextCursor(cursor);
-    if (parent->highlighter()) {
-        parent->highlighter()->rehighlight();
+    q->setTextCursor(cursor);
+    if (q->highlighter()) {
+        q->highlighter()->rehighlight();
     }
 }
 
-void KTextEdit::Private::toggleAutoSpellCheck()
+void KTextEditPrivate::toggleAutoSpellCheck()
 {
-    parent->setCheckSpellingEnabled(!parent->checkSpellingEnabled());
+    Q_Q(KTextEdit);
+
+    q->setCheckSpellingEnabled(!q->checkSpellingEnabled());
 }
 
-void KTextEdit::Private::undoableClear()
+void KTextEditPrivate::undoableClear()
 {
-    QTextCursor cursor = parent->textCursor();
+    Q_Q(KTextEdit);
+
+    QTextCursor cursor = q->textCursor();
     cursor.beginEditBlock();
     cursor.movePosition(QTextCursor::Start);
     cursor.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
@@ -259,15 +158,19 @@ void KTextEdit::Private::undoableClear()
     cursor.endEditBlock();
 }
 
-void KTextEdit::Private::slotAllowTab()
+void KTextEditPrivate::slotAllowTab()
 {
-    parent->setTabChangesFocus(!parent->tabChangesFocus());
+    Q_Q(KTextEdit);
+
+    q->setTabChangesFocus(!q->tabChangesFocus());
 }
 
-void KTextEdit::Private::menuActivated(QAction *action)
+void KTextEditPrivate::menuActivated(QAction *action)
 {
+    Q_Q(KTextEdit);
+
     if (action == spellCheckAction) {
-        parent->checkSpelling();
+        q->checkSpelling();
     } else if (action == autoSpellCheckAction) {
         toggleAutoSpellCheck();
     } else if (action == allowTab) {
@@ -275,42 +178,47 @@ void KTextEdit::Private::menuActivated(QAction *action)
     }
 }
 
-void KTextEdit::Private::slotFindHighlight(const QString &text, int matchingIndex, int matchingLength)
+void KTextEditPrivate::slotFindHighlight(const QString &text, int matchingIndex, int matchingLength)
 {
+    Q_Q(KTextEdit);
+
     Q_UNUSED(text)
-    //qDebug() << "Highlight: [" << text << "] mi:" << matchingIndex << " ml:" << matchingLength;
-    QTextCursor tc = parent->textCursor();
+    // qDebug() << "Highlight: [" << text << "] mi:" << matchingIndex << " ml:" << matchingLength;
+    QTextCursor tc = q->textCursor();
     tc.setPosition(matchingIndex);
     tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchingLength);
-    parent->setTextCursor(tc);
-    parent->ensureCursorVisible();
+    q->setTextCursor(tc);
+    q->ensureCursorVisible();
 }
 
-void KTextEdit::Private::slotReplaceText(const QString &text, int replacementIndex, int replacedLength, int matchedLength)
+void KTextEditPrivate::slotReplaceText(const QString &text, int replacementIndex, int replacedLength, int matchedLength)
 {
-    //qDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength;
-    QTextCursor tc = parent->textCursor();
+    Q_Q(KTextEdit);
+
+    // qDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength;
+    QTextCursor tc = q->textCursor();
     tc.setPosition(replacementIndex);
     tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchedLength);
     tc.removeSelectedText();
     tc.insertText(text.mid(replacementIndex, replacedLength));
     if (replace->options() & KReplaceDialog::PromptOnReplace) {
-        parent->setTextCursor(tc);
-        parent->ensureCursorVisible();
+        q->setTextCursor(tc);
+        q->ensureCursorVisible();
     }
     lastReplacedPosition = replacementIndex;
 }
 
-void KTextEdit::Private::init()
+void KTextEditPrivate::init()
 {
-    KCursor::setAutoHideCursor(parent, true, false);
-    parent->connect(parent, &KTextEdit::languageChanged,
-                    parent, &KTextEdit::setSpellCheckingLanguage);
+    Q_Q(KTextEdit);
+
+    KCursor::setAutoHideCursor(q, true, false);
+    q->connect(q, &KTextEdit::languageChanged, q, &KTextEdit::setSpellCheckingLanguage);
 }
 
-KTextDecorator::KTextDecorator(KTextEdit *textEdit):
-    SpellCheckDecorator(textEdit),
-    m_textEdit(textEdit)
+KTextDecorator::KTextDecorator(KTextEdit *textEdit)
+    : SpellCheckDecorator(textEdit)
+    , m_textEdit(textEdit)
 {
 }
 
@@ -320,29 +228,46 @@ bool KTextDecorator::isSpellCheckingEnabledForBlock(const QString &textBlock) co
 }
 
 KTextEdit::KTextEdit(const QString &text, QWidget *parent)
-    : QTextEdit(text, parent), d(new Private(this))
+    : KTextEdit(*new KTextEditPrivate(this), text, parent)
 {
+}
+
+KTextEdit::KTextEdit(KTextEditPrivate &dd, const QString &text, QWidget *parent)
+    : QTextEdit(text, parent)
+    , d(&dd)
+{
+    Q_D(KTextEdit);
+
     d->init();
 }
 
 KTextEdit::KTextEdit(QWidget *parent)
-    : QTextEdit(parent), d(new Private(this))
+    : KTextEdit(*new KTextEditPrivate(this), parent)
 {
+}
+
+KTextEdit::KTextEdit(KTextEditPrivate &dd, QWidget *parent)
+    : QTextEdit(parent)
+    , d(&dd)
+{
+    Q_D(KTextEdit);
+
     d->init();
 }
 
-KTextEdit::~KTextEdit()
-{
-    delete d;
-}
+KTextEdit::~KTextEdit() = default;
 
 const QString &KTextEdit::spellCheckingLanguage() const
 {
+    Q_D(const KTextEdit);
+
     return d->spellCheckingLanguage;
 }
 
 void KTextEdit::setSpellCheckingLanguage(const QString &_language)
 {
+    Q_D(KTextEdit);
+
     if (highlighter()) {
         highlighter()->setCurrentLanguage(_language);
         highlighter()->rehighlight();
@@ -350,12 +275,14 @@ void KTextEdit::setSpellCheckingLanguage(const QString &_language)
 
     if (_language != d->spellCheckingLanguage) {
         d->spellCheckingLanguage = _language;
-        emit languageChanged(_language);
+        Q_EMIT languageChanged(_language);
     }
 }
 
 void KTextEdit::showSpellConfigDialog(const QString &windowIcon)
 {
+    Q_D(KTextEdit);
+
     Sonnet::ConfigDialog dialog(this);
     if (!d->spellCheckingLanguage.isEmpty()) {
         dialog.setLanguage(d->spellCheckingLanguage);
@@ -370,6 +297,8 @@ void KTextEdit::showSpellConfigDialog(const QString &windowIcon)
 
 bool KTextEdit::event(QEvent *ev)
 {
+    Q_D(KTextEdit);
+
     if (ev->type() == QEvent::ShortcutOverride) {
         QKeyEvent *e = static_cast<QKeyEvent *>(ev);
         if (d->overrideShortcut(e)) {
@@ -380,123 +309,127 @@ bool KTextEdit::event(QEvent *ev)
     return QTextEdit::event(ev);
 }
 
-bool KTextEdit::Private::handleShortcut(const QKeyEvent *event)
+bool KTextEditPrivate::handleShortcut(const QKeyEvent *event)
 {
+    Q_Q(KTextEdit);
+
     const int key = event->key() | event->modifiers();
 
     if (KStandardShortcut::copy().contains(key)) {
-        parent->copy();
+        q->copy();
         return true;
     } else if (KStandardShortcut::paste().contains(key)) {
-        parent->paste();
+        q->paste();
         return true;
     } else if (KStandardShortcut::cut().contains(key)) {
-        parent->cut();
+        q->cut();
         return true;
     } else if (KStandardShortcut::undo().contains(key)) {
-        if (!parent->isReadOnly()) {
-            parent->undo();
+        if (!q->isReadOnly()) {
+            q->undo();
         }
         return true;
     } else if (KStandardShortcut::redo().contains(key)) {
-        if (!parent->isReadOnly()) {
-            parent->redo();
+        if (!q->isReadOnly()) {
+            q->redo();
         }
         return true;
     } else if (KStandardShortcut::deleteWordBack().contains(key)) {
-        if (!parent->isReadOnly()) {
-           parent->deleteWordBack();
+        if (!q->isReadOnly()) {
+            q->deleteWordBack();
         }
         return true;
     } else if (KStandardShortcut::deleteWordForward().contains(key)) {
-        if (!parent->isReadOnly()) {
-            parent->deleteWordForward();
+        if (!q->isReadOnly()) {
+            q->deleteWordForward();
         }
         return true;
     } else if (KStandardShortcut::backwardWord().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
-        cursor.movePosition(QTextCursor::PreviousWord);
-        parent->setTextCursor(cursor);
+        QTextCursor cursor = q->textCursor();
+        // We use visual positioning here since keyboard arrows represents visual direction (left, right)
+        cursor.movePosition(QTextCursor::WordLeft);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::forwardWord().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
-        cursor.movePosition(QTextCursor::NextWord);
-        parent->setTextCursor(cursor);
+        QTextCursor cursor = q->textCursor();
+        // We use visual positioning here since keyboard arrows represents visual direction (left, right)
+        cursor.movePosition(QTextCursor::WordRight);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::next().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         bool moved = false;
-        qreal lastY = parent->cursorRect(cursor).bottom();
+        qreal lastY = q->cursorRect(cursor).bottom();
         qreal distance = 0;
         do {
-            qreal y = parent->cursorRect(cursor).bottom();
+            qreal y = q->cursorRect(cursor).bottom();
             distance += qAbs(y - lastY);
             lastY = y;
             moved = cursor.movePosition(QTextCursor::Down);
-        } while (moved && distance < parent->viewport()->height());
+        } while (moved && distance < q->viewport()->height());
 
         if (moved) {
             cursor.movePosition(QTextCursor::Up);
-            parent->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepAdd);
+            q->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepAdd);
         }
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::prior().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         bool moved = false;
-        qreal lastY = parent->cursorRect(cursor).bottom();
+        qreal lastY = q->cursorRect(cursor).bottom();
         qreal distance = 0;
         do {
-            qreal y = parent->cursorRect(cursor).bottom();
+            qreal y = q->cursorRect(cursor).bottom();
             distance += qAbs(y - lastY);
             lastY = y;
             moved = cursor.movePosition(QTextCursor::Up);
-        } while (moved && distance < parent->viewport()->height());
+        } while (moved && distance < q->viewport()->height());
 
         if (moved) {
             cursor.movePosition(QTextCursor::Down);
-            parent->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepSub);
+            q->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepSub);
         }
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::begin().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         cursor.movePosition(QTextCursor::Start);
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::end().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         cursor.movePosition(QTextCursor::End);
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::beginningOfLine().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         cursor.movePosition(QTextCursor::StartOfLine);
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (KStandardShortcut::endOfLine().contains(key)) {
-        QTextCursor cursor = parent->textCursor();
+        QTextCursor cursor = q->textCursor();
         cursor.movePosition(QTextCursor::EndOfLine);
-        parent->setTextCursor(cursor);
+        q->setTextCursor(cursor);
         return true;
     } else if (findReplaceEnabled && KStandardShortcut::find().contains(key)) {
-        parent->slotFind();
+        q->slotFind();
         return true;
     } else if (findReplaceEnabled && KStandardShortcut::findNext().contains(key)) {
-        parent->slotFindNext();
+        q->slotFindNext();
         return true;
     } else if (findReplaceEnabled && KStandardShortcut::findPrev().contains(key)) {
-        parent->slotFindPrevious();
+        q->slotFindPrevious();
         return true;
     } else if (findReplaceEnabled && KStandardShortcut::replace().contains(key)) {
-        if (!parent->isReadOnly()) {
-            parent->slotReplace();
+        if (!q->isReadOnly()) {
+            q->slotReplace();
         }
         return true;
     } else if (KStandardShortcut::pasteSelection().contains(key)) {
         QString text = QApplication::clipboard()->text(QClipboard::Selection);
         if (!text.isEmpty()) {
-            parent->insertPlainText(text);    // TODO: check if this is html? (MiB)
+            q->insertPlainText(text); // TODO: check if this is html? (MiB)
         }
         return true;
     }
@@ -512,22 +445,29 @@ static void deleteWord(QTextCursor cursor, QTextCursor::MoveOperation op)
 
 void KTextEdit::deleteWordBack()
 {
+    // We use logical positioning here since deleting should always delete the previous word
+    // (left in case of LTR text, right in case of RTL text)
     deleteWord(textCursor(), QTextCursor::PreviousWord);
 }
 
 void KTextEdit::deleteWordForward()
 {
-    deleteWord(textCursor(), QTextCursor::WordRight);
+    // We use logical positioning here since deleting should always delete the previous word
+    // (left in case of LTR text, right in case of RTL text)
+    deleteWord(textCursor(), QTextCursor::NextWord);
 }
 
 QMenu *KTextEdit::mousePopupMenu()
 {
+    Q_D(KTextEdit);
+
     QMenu *popup = createStandardContextMenu();
     if (!popup) {
         return nullptr;
     }
-    connect(popup, SIGNAL(triggered(QAction*)),
-            this, SLOT(menuActivated(QAction*)));
+    connect(popup, &QMenu::triggered, this, [d](QAction *action) {
+        d->menuActivated(action);
+    });
 
     const bool emptyDocument = document()->isEmpty();
     if (!isReadOnly()) {
@@ -538,8 +478,13 @@ QMenu *KTextEdit::mousePopupMenu()
         if (idx < actionList.count()) {
             separatorAction = actionList.at(idx);
         }
+
+        auto undoableClearSlot = [d]() {
+            d->undoableClear();
+        };
+
         if (separatorAction) {
-            QAction *clearAllAction = KStandardAction::clear(this, SLOT(undoableClear()), popup);
+            QAction *clearAllAction = KStandardAction::clear(this, undoableClearSlot, popup);
             if (emptyDocument) {
                 clearAllAction->setEnabled(false);
             }
@@ -553,9 +498,7 @@ QMenu *KTextEdit::mousePopupMenu()
             d->speller = new Sonnet::Speller();
         }
         if (!d->speller->availableBackends().isEmpty()) {
-
-            d->spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")),
-                                                   i18n("Check Spelling..."));
+            d->spellCheckAction = popup->addAction(QIcon::fromTheme(QStringLiteral("tools-check-spelling")), i18n("Check Spelling..."));
             if (emptyDocument) {
                 d->spellCheckAction->setEnabled(false);
             }
@@ -571,12 +514,10 @@ QMenu *KTextEdit::mousePopupMenu()
 
                     QAction *languageAction = d->languagesMenu->addAction(i.key());
                     languageAction->setCheckable(true);
-                    languageAction->setChecked(language == i.value() || (language.isEmpty()
-                                                                         && d->speller->defaultLanguage() == i.value()));
+                    languageAction->setChecked(language == i.value() || (language.isEmpty() && d->speller->defaultLanguage() == i.value()));
                     languageAction->setData(i.value());
                     languageAction->setActionGroup(languagesGroup);
-                    connect(languageAction, &QAction::triggered,
-                            [this, languageAction]() {
+                    connect(languageAction, &QAction::triggered, [this, languageAction]() {
                         setSpellCheckingLanguage(languageAction->data().toString());
                     });
                 }
@@ -596,9 +537,9 @@ QMenu *KTextEdit::mousePopupMenu()
     }
 
     if (d->findReplaceEnabled) {
-        QAction *findAction = KStandardAction::find(this, SLOT(slotFind()), popup);
-        QAction *findNextAction = KStandardAction::findNext(this, SLOT(slotFindNext()), popup);
-        QAction *findPrevAction = KStandardAction::findPrev(this, SLOT(slotFindPrevious()), popup);
+        QAction *findAction = KStandardAction::find(this, &KTextEdit::slotFind, popup);
+        QAction *findNextAction = KStandardAction::findNext(this, &KTextEdit::slotFindNext, popup);
+        QAction *findPrevAction = KStandardAction::findPrev(this, &KTextEdit::slotFindPrevious, popup);
         if (emptyDocument) {
             findAction->setEnabled(false);
             findNextAction->setEnabled(false);
@@ -613,7 +554,7 @@ QMenu *KTextEdit::mousePopupMenu()
         popup->addAction(findPrevAction);
 
         if (!isReadOnly()) {
-            QAction *replaceAction = KStandardAction::replace(this, SLOT(slotReplace()), popup);
+            QAction *replaceAction = KStandardAction::replace(this, &KTextEdit::slotReplace, popup);
             if (emptyDocument) {
                 replaceAction->setEnabled(false);
             }
@@ -632,6 +573,8 @@ QMenu *KTextEdit::mousePopupMenu()
 
 void KTextEdit::slotSpeakText()
 {
+    Q_D(KTextEdit);
+
 #ifdef HAVE_SPEECH
     QString text;
     if (textCursor().hasSelection()) {
@@ -663,6 +606,8 @@ void KTextEdit::createHighlighter()
 
 Sonnet::Highlighter *KTextEdit::highlighter() const
 {
+    Q_D(const KTextEdit);
+
     if (d->decorator) {
         return d->decorator->highlighter();
     } else {
@@ -672,12 +617,16 @@ Sonnet::Highlighter *KTextEdit::highlighter() const
 
 void KTextEdit::clearDecorator()
 {
+    Q_D(KTextEdit);
+
     delete d->decorator;
     d->decorator = nullptr;
 }
 
 void KTextEdit::addTextDecorator(Sonnet::SpellCheckDecorator *decorator)
 {
+    Q_D(KTextEdit);
+
     d->decorator = decorator;
 }
 
@@ -688,20 +637,22 @@ void KTextEdit::setHighlighter(Sonnet::Highlighter *_highLighter)
     delete decorator->highlighter();
     decorator->setHighlighter(_highLighter);
 
-    //KTextEdit used to take ownership of the highlighter, Sonnet::SpellCheckDecorator does not.
-    //so we reparent the highlighter so it will be deleted when the decorator is destroyed
+    // KTextEdit used to take ownership of the highlighter, Sonnet::SpellCheckDecorator does not.
+    // so we reparent the highlighter so it will be deleted when the decorator is destroyed
     _highLighter->setParent(decorator);
     addTextDecorator(decorator);
 }
 
 void KTextEdit::setCheckSpellingEnabled(bool check)
 {
-    emit checkSpellingChanged(check);
+    Q_D(KTextEdit);
+
+    Q_EMIT checkSpellingChanged(check);
     if (check == d->spellCheckingEnabled) {
         return;
     }
 
-    // From the above statment we know know that if we're turning checking
+    // From the above statement we now know that if we're turning checking
     // on that we need to create a new highlighter and if we're turning it
     // off we should remove the old one.
 
@@ -720,6 +671,8 @@ void KTextEdit::setCheckSpellingEnabled(bool check)
 
 void KTextEdit::focusInEvent(QFocusEvent *event)
 {
+    Q_D(KTextEdit);
+
     if (d->spellCheckingEnabled && !isReadOnly() && !d->decorator) {
         createHighlighter();
     }
@@ -729,6 +682,8 @@ void KTextEdit::focusInEvent(QFocusEvent *event)
 
 bool KTextEdit::checkSpellingEnabled() const
 {
+    Q_D(const KTextEdit);
+
     return d->spellCheckingEnabled;
 }
 
@@ -739,6 +694,8 @@ bool KTextEdit::shouldBlockBeSpellChecked(const QString &) const
 
 void KTextEdit::setReadOnly(bool readOnly)
 {
+    Q_D(KTextEdit);
+
     if (!readOnly && hasFocus() && d->spellCheckingEnabled && !d->decorator) {
         createHighlighter();
     }
@@ -774,11 +731,15 @@ void KTextEdit::setReadOnly(bool readOnly)
 
 void KTextEdit::checkSpelling()
 {
+    Q_D(KTextEdit);
+
     d->checkSpelling(false);
 }
 
 void KTextEdit::forceSpellChecking()
 {
+    Q_D(KTextEdit);
+
     d->checkSpelling(true);
 }
 
@@ -793,15 +754,16 @@ void KTextEdit::highlightWord(int length, int pos)
 
 void KTextEdit::replace()
 {
-    if (document()->isEmpty()) {  // saves having to track the text changes
+    Q_D(KTextEdit);
+
+    if (document()->isEmpty()) { // saves having to track the text changes
         return;
     }
 
     if (d->repDlg) {
         d->repDlg->activateWindow();
     } else {
-        d->repDlg = new KReplaceDialog(this, 0,
-                                       QStringList(), QStringList(), false);
+        d->repDlg = new KReplaceDialog(this, 0, QStringList(), QStringList(), false);
         connect(d->repDlg, &KFindDialog::okClicked, this, &KTextEdit::slotDoReplace);
     }
     d->repDlg->show();
@@ -809,6 +771,8 @@ void KTextEdit::replace()
 
 void KTextEdit::slotDoReplace()
 {
+    Q_D(KTextEdit);
+
     if (!d->repDlg) {
         // Should really assert()
         return;
@@ -828,13 +792,24 @@ void KTextEdit::slotDoReplace()
         d->repIndex = textCursor().anchor();
     }
 
-    // Connect highlight signal to code which handles highlighting
-    // of found text.
-    connect(d->replace, SIGNAL(highlight(QString,int,int)),
-            this, SLOT(slotFindHighlight(QString,int,int)));
+    // Connect textFound signal to code which handles highlighting of found text.
+    connect(d->replace, &KFind::textFound, this, [d](const QString &text, int matchingIndex, int matchedLength) {
+        d->slotFindHighlight(text, matchingIndex, matchedLength);
+    });
     connect(d->replace, &KFind::findNext, this, &KTextEdit::slotReplaceNext);
-    connect(d->replace, SIGNAL(replace(QString,int,int,int)),
-            this, SLOT(slotReplaceText(QString,int,int,int)));
+
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 81)
+    connect(d->replace,
+            qOverload<const QString &, int, int, int>(&KReplace::replace),
+            this,
+            [d](const QString &text, int replacementIndex, int replacedLength, int matchedLength) {
+                d->slotReplaceText(text, replacementIndex, replacedLength, matchedLength);
+            });
+#else
+    connect(d->replace, &KReplace::textReplaced, this, [d](const QString &text, int replacementIndex, int replacedLength, int matchedLength) {
+        d->slotReplaceText(text, replacementIndex, replacedLength, matchedLength);
+    });
+#endif
 
     d->repDlg->close();
     slotReplaceNext();
@@ -842,6 +817,8 @@ void KTextEdit::slotDoReplace()
 
 void KTextEdit::slotReplaceNext()
 {
+    Q_D(KTextEdit);
+
     if (!d->replace) {
         return;
     }
@@ -875,14 +852,16 @@ void KTextEdit::slotReplaceNext()
         d->replace->deleteLater(); // we are in a slot connected to m_replace, don't delete it right away
         d->replace = nullptr;
         ensureCursorVisible();
-        //or           if ( m_replace->shouldRestart() ) { reinit (w/o FromCursor) and call slotReplaceNext(); }
+        // or           if ( m_replace->shouldRestart() ) { reinit (w/o FromCursor) and call slotReplaceNext(); }
     } else {
-        //m_replace->closeReplaceNextDialog();
+        // m_replace->closeReplaceNextDialog();
     }
 }
 
 void KTextEdit::slotDoFind()
 {
+    Q_D(KTextEdit);
+
     if (!d->findDlg) {
         // Should really assert()
         return;
@@ -899,10 +878,10 @@ void KTextEdit::slotDoFind()
         d->findIndex = textCursor().anchor();
     }
 
-    // Connect highlight signal to code which handles highlighting
-    // of found text.
-    connect(d->find, SIGNAL(highlight(QString,int,int)),
-            this, SLOT(slotFindHighlight(QString,int,int)));
+    // Connect textFound() signal to code which handles highlighting of found text
+    connect(d->find, &KFind::textFound, this, [d](const QString &text, int matchingIndex, int matchedLength) {
+        d->slotFindHighlight(text, matchingIndex, matchedLength);
+    });
     connect(d->find, &KFind::findNext, this, &KTextEdit::slotFindNext);
 
     d->findDlg->close();
@@ -912,6 +891,8 @@ void KTextEdit::slotDoFind()
 
 void KTextEdit::slotFindNext()
 {
+    Q_D(KTextEdit);
+
     if (!d->find) {
         return;
     }
@@ -932,14 +913,16 @@ void KTextEdit::slotFindNext()
         d->find->disconnect(this);
         d->find->deleteLater(); // we are in a slot connected to m_find, don't delete right away
         d->find = nullptr;
-        //or           if ( m_find->shouldRestart() ) { reinit (w/o FromCursor) and call slotFindNext(); }
+        // or           if ( m_find->shouldRestart() ) { reinit (w/o FromCursor) and call slotFindNext(); }
     } else {
-        //m_find->closeFindNextDialog();
+        // m_find->closeFindNextDialog();
     }
 }
 
 void KTextEdit::slotFindPrevious()
 {
+    Q_D(KTextEdit);
+
     if (!d->find) {
         return;
     }
@@ -953,7 +936,9 @@ void KTextEdit::slotFindPrevious()
 
 void KTextEdit::slotFind()
 {
-    if (document()->isEmpty()) {  // saves having to track the text changes
+    Q_D(KTextEdit);
+
+    if (document()->isEmpty()) { // saves having to track the text changes
         return;
     }
 
@@ -968,15 +953,16 @@ void KTextEdit::slotFind()
 
 void KTextEdit::slotReplace()
 {
-    if (document()->isEmpty()) {  // saves having to track the text changes
+    Q_D(KTextEdit);
+
+    if (document()->isEmpty()) { // saves having to track the text changes
         return;
     }
 
     if (d->repDlg) {
         d->repDlg->activateWindow();
     } else {
-        d->repDlg = new KReplaceDialog(this, 0,
-                                       QStringList(), QStringList(), false);
+        d->repDlg = new KReplaceDialog(this, 0, QStringList(), QStringList(), false);
         connect(d->repDlg, &KFindDialog::okClicked, this, &KTextEdit::slotDoReplace);
     }
     d->repDlg->show();
@@ -984,15 +970,19 @@ void KTextEdit::slotReplace()
 
 void KTextEdit::enableFindReplace(bool enabled)
 {
+    Q_D(KTextEdit);
+
     d->findReplaceEnabled = enabled;
 }
 
 void KTextEdit::showTabAction(bool show)
 {
+    Q_D(KTextEdit);
+
     d->showTabAction = show;
 }
 
-bool KTextEdit::Private::overrideShortcut(const QKeyEvent *event)
+bool KTextEditPrivate::overrideShortcut(const QKeyEvent *event)
 {
     const int key = event->key() | event->modifiers();
 
@@ -1044,6 +1034,8 @@ bool KTextEdit::Private::overrideShortcut(const QKeyEvent *event)
 
 void KTextEdit::keyPressEvent(QKeyEvent *event)
 {
+    Q_D(KTextEdit);
+
     if (d->handleShortcut(event)) {
         event->accept();
     } else {
@@ -1053,6 +1045,8 @@ void KTextEdit::keyPressEvent(QKeyEvent *event)
 
 void KTextEdit::showAutoCorrectButton(bool show)
 {
+    Q_D(KTextEdit);
+
     d->showAutoCorrectionButton = show;
 }
 
